@@ -1,8 +1,8 @@
 /*
  * Test suite for streaming data from the server.
  *
- * Written by Russ Allbery <rra@stanford.edu>
- * Copyright 2006, 2009, 2010, 2012, 2013
+ * Written by Russ Allbery <eagle@eyrie.org>
+ * Copyright 2006, 2009, 2010, 2012, 2013, 2014
  *     The Board of Trustees of the Leland Stanford Junior University
  *
  * See LICENSE for licensing terms.
@@ -16,6 +16,7 @@
 #include <tests/tap/basic.h>
 #include <tests/tap/kerberos.h>
 #include <tests/tap/remctl.h>
+#include <util/protocol.h>
 
 
 int
@@ -32,7 +33,7 @@ main(void)
     config = kerberos_setup(TAP_KRB_NEEDS_KEYTAB);
     remctld_start(config, "data/conf-simple", NULL);
 
-    plan(35);
+    plan(45);
 
     /* First, version 2. */
     r = remctl_new();
@@ -105,7 +106,6 @@ main(void)
                 diag("%.*s", (int) output->length, output->data);
         } else if (output->type == REMCTL_OUT_STATUS) {
             is_int(0, output->status, "...correct exit status");
-            is_int(1728361, total, "...correct total size");
             break;
         } else {
             is_int(REMCTL_OUT_OUTPUT, output->type, "Incorrect output type");
@@ -114,6 +114,7 @@ main(void)
         }
         output = remctl_output(r);
     }
+    is_int(1728361, total, "...correct total size");
     remctl_close(r);
 
     /* Now, version 1. */
@@ -149,5 +150,36 @@ main(void)
     }
     remctl_close(r);
 
+    /*
+     * Test with more data than will fit into a single token.  Since protocol
+     * one does not support multiple tokens, this requires truncation, but we
+     * should still get the correct exit status.
+     */
+    r = remctl_new();
+    ok(r != NULL, "remctl_new protocol version 1");
+    if (r == NULL)
+        bail("remctl_new returned NULL");
+    r->protocol = 1;
+    ok(remctl_open(r, "localhost", 14373, config->principal), "remctl_open");
+    ok(remctl_command(r, command_cat), "remctl_command cat");
+    output = remctl_output(r);
+    if (output == NULL)
+        ok_block(4, false, "output is not null");
+    else {
+        ok(output != NULL, "output is not null");
+        is_int(REMCTL_OUT_OUTPUT, output->type, "...and is right type");
+        is_int(TOKEN_MAX_OUTPUT_V1, output->length, "...and right length");
+        is_int(1, output->stream, "...and right stream");
+    }
+    output = remctl_output(r);
+    if (output == NULL)
+        ok_block(3, false, "status token is not null");
+    else {
+        ok(output != NULL, "status token is not null");
+        is_int(REMCTL_OUT_STATUS, output->type, "...and is right type");
+        is_int(0, output->status, "...and is right status");
+    }
+    remctl_close(r);
+
     return 0;
 }
