@@ -8,7 +8,7 @@
  * which can be found at <http://www.eyrie.org/~eagle/software/rra-c-util/>.
  *
  * Written by Russ Allbery <eagle@eyrie.org>
- * Copyright 2006, 2007, 2009, 2011, 2012, 2013
+ * Copyright 2006, 2007, 2009, 2011, 2012, 2013, 2014
  *     The Board of Trustees of the Leland Stanford Junior University
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -88,7 +88,7 @@ remctld_start_internal(struct kerberos_config *krbconf, const char *config,
     while ((arg = va_arg(args_copy, const char *)) != NULL)
         length++;
     va_end(args_copy);
-    argv = bmalloc(length * sizeof(const char *));
+    argv = bcalloc(length, sizeof(const char *));
     i = 0;
     argv[i++] = path_remctld;
     argv[i++] = "-mdSF";
