@@ -58,11 +58,14 @@
 
 /*
  * If we're running the test suite, call testing versions of the write
- * functions.  #undef pwrite first because large file support may define a
- * macro pwrite (pointing to pwrite64) on some platforms (e.g. Solaris).
+ * functions.  #undef the functions first since large file support may define
+ * a macro pwrite (pointing to pwrite64) on some platforms (e.g. Solaris),
+ * and it's possible the other functions may be similarly affected.
  */
 #if TESTING
 # undef pwrite
+# undef write
+# undef writev
 # define pwrite fake_pwrite
 # define write  fake_write
 # define writev fake_writev
@@ -77,7 +80,7 @@ xwrite(int fd, const void *buffer, size_t size)
 {
     size_t total;
     ssize_t status;
-    int count = 0;
+    unsigned int count = 0;
 
     if (size == 0)
 	return 0;
@@ -105,7 +108,7 @@ xpwrite(int fd, const void *buffer, size_t size, off_t offset)
 {
     size_t total;
     ssize_t status;
-    int count = 0;
+    unsigned int count = 0;
 
     if (size == 0)
 	return 0;
@@ -134,7 +137,7 @@ xwritev(int fd, const struct iovec iov[], int iovcnt)
 {
     ssize_t total, status = 0;
     size_t left, offset;
-    int iovleft, i, count;
+    unsigned int iovleft, i, count;
     struct iovec *tmpiov;
 
     /*
@@ -150,7 +153,7 @@ xwritev(int fd, const struct iovec iov[], int iovcnt)
     }
 
     /* Get a count of the total number of bytes in the iov array. */
-    for (total = 0, i = 0; i < iovcnt; i++)
+    for (total = 0, i = 0; i < (unsigned int) iovcnt; i++)
         total += iov[i].iov_len;
     if (total == 0)
 	return 0;
